(*----------------------------------------------------------------------*)
(*        PIBUPLOAD.PAS --- File Upload Routines for PibTerm            *)
(*----------------------------------------------------------------------*)
(*                                                                      *)
(*  Author:  Philip R. Burns                                            *)
(*           Copyright (c) October, 1985                                *)
(*                                                                      *)
(*  Version: 1.0   (January, 1985)                                      *)
(*           2.0   (June, 1985)                                         *)
(*           3.0   (October, 1985)                                      *)
(*                                                                      *)
(*  Systems: For MS-DOS on IBM PCs and close compatibles only.          *)
(*           Note:  I have checked these on Zenith 151s under           *)
(*                  MSDOS 2.1 and IBM PCs under PCDOS 2.0.              *)
(*                                                                      *)
(*  History: Original with me, but the XMODEM is based upon the         *)
(*           famous (X)MODEM(7) programs of Christiansen, et. al.       *)
(*           Note that both the Checksum and CRC versions of XMODEM     *)
(*           are available here.                                        *)
(*                                                                      *)
(*           Version 2.0 added the MODEM7, TELINK, and YMODEM protocols.*)
(*           Version 3.0 added the Kermit protocol.                     *)
(*           Parts of the Kermit code are based upon a CP/M Turbo       *)
(*           version of Kermit by Jeff Duncan.                          *)
(*                                                                      *)
(*           Suggestions for improvements or corrections are welcome.   *)
(*           Please leave messages on Gene Plantz's BBS (312) 882 4145  *)
(*           or Ron Fox's BBS (312) 940 6496.                           *)
(*                                                                      *)
(*           IF you use this code in your own programs, please be nice  *)
(*           and give proper credit.                                    *)
(*                                                                      *)
(*----------------------------------------------------------------------*)
(*                                                                      *)
(*                            Restriction                               *)
(*                            -----------                               *)
(*                                                                      *)
(*           You may use this code only for NON COMMERCIAL purposes     *)
(*           unless you explicitly obtain my permission.  I take a dim  *)
(*           view of others making money on my work and those of other  *)
(*           people whose code I've inserted here.                      *)
(*                                                                      *)
(*----------------------------------------------------------------------*)
(*                                                                      *)
(*  Routines:                                                           *)
(*                                                                      *)
(*     PibUpLoad  --- Main control routine                              *)
(*                                                                      *)
(*     Send_Ascii_File     --- Send Ascii file to another computer      *)
(*     Send_Xmodem_File    --- Send file with XMODEM                    *)
(*     Send_Modem7_File    --- Send file with MODEM7                    *)
(*     Send_Telink_File    --- Send file with Telink                    *)
(*     Send_Kermit_File    --- Send file with Kermit                    *)
(*                                                                      *)
(*     Get_Upload_Transfer_Protocol --- Determines type of transfer     *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

(*----------------------------------------------------------------------*)
(*           PibUpLoad --- Control routine for uploads/downloads        *)
(*----------------------------------------------------------------------*)

OVERLAY PROCEDURE PibUpLoad( Transfer_Protocol : Transfer_Type );

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*     Procedure:  PibUpload                                            *)
(*                                                                      *)
(*     Purpose:    Controls uploading of files to remote hosts.         *)
(*                                                                      *)
(*     Calling Sequence:                                                *)
(*                                                                      *)
(*        PibUpLoad( Transfer_Protocol : Transfer_Type );               *)
(*                                                                      *)
(*           Transfer_Protocol --- type of transfer to do               *)
(*                                                                      *)
(*     Remarks:                                                         *)
(*                                                                      *)
(*        Currently, the only available protocols are:                  *)
(*                                                                      *)
(*           Ascii file transfer (no error-correction)                  *)
(*           Xmodem with Checksum                                       *)
(*           Xmodem with CRC                                            *)
(*           Modem7                                                     *)
(*           Telink                                                     *)
(*           Kermit                                                     *)
(*                                                                      *)
(*      Calls:   Send_Ascii_File                                        *)
(*               Send_Xmodem_File                                       *)
(*               Send_Modem7_File                                       *)
(*               Send_Kermit_File                                       *)
(*               Send_Telink_File                                       *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR
   OK_Init           : BOOLEAN;
   I                 : INTEGER;